#ifndef     __STANDARD_H
#define     __STANDARD_H

/* Special key conbinations */

#define ESC 0x1b
#define ENT 0x0d
#define F1  0x3b
#define F2  0x3c
#define F3  0x3d
#define F4  0x3e
#define F5  0x3f
#define F6  0x40
#define F7  0x41
#define F8  0x42
#define F9  0x43
#define F10 0x44
#define ALT_T 0x14
#define ALT_H 0x23
#define ALT_D 0x20
#define ALT_M 0x32
#define ALT_Q 0x10
#define ALT_W 0x11
#define ALT_C 0x2E
#define ALT_U 0x16
#define AUP    0x48
#define ALEFT  0x4b
#define ADOWN  0x50
#define ARIGHT 0x4d
#define HOME   0x47
#define BKSP   0x08
#define CTL_T  0x14
#define CTL_H  0x08
#define CTL_D  0x04
#define CTL_M  0x0D
#define CTL_Q  0x11
#define CTL_W  0x17
#define CTL_C  0x03
#define CTL_U  0x15
#define CTL_A  0x01


/*  Logical operators and values  */

#define     AND     &&
#define     NOT     !
#define     OR      ||
#define     EQU     ==
#define     NOTEQU  !=
#define     LESSEQU <=
#define     MOREEQU >=
#define     NEG     ~


#define     FALSE   0
#define     TRUE    1	    // for consistency with TRUE = NOT FALSE

#define     OFF     0
#define     ON	     1

/*  Case conversion standards */

#define ctoupper(c) ((c) + 'A' - 'a')

/*  Convenient data types  */

typedef unsigned char	BYTE;
typedef unsigned int 	WORD;
typedef unsigned long	DWORD;

typedef signed char	   SBYTE;
typedef signed int	   SWORD;
typedef signed long	   SDWORD;

typedef unsigned char	CHAR;

typedef void (*fptr)(int);

//typedef int		         BOOL;

/*  Macros to decompose 8-bit,16-bit and 32-bit objects into high and low
	 components and to reconstitute them  */

#define HIGHNIBBLE(x) (((BYTE)((BYTE)(x) >> 4 )) & 0x0F)
#define LOWNIBBLE(x)  ((BYTE)((BYTE)(x) & 0x0F))
#define MK_BYTE(high,low) (((BYTE) (high) << 4) | (low))

#define HIGHBYTE(x) ((BYTE) ((x) >> 8))
#define LOWBYTE(x)  ((BYTE) (x))
#define MK_WORD(high,low)  (((WORD) (high) << 8) | (low))

#define HIGHWORD(x) ((WORD) ((x) >> 16))
#define LOWWORD(x)  ((WORD) (x))
#define MK_DWORD(high,low) (((DWORD) (high) << 16) | (low))

#define CVT_FPDW( x ) (((DWORD)FP_SEG(x)<<16)+FP_OFF(x));
#define CVT_DWFP( x ) FP_SEG(x)=HIGHWORD(x);FP_OFF(x)=LOWWORD(x);

/* define some memory function macros */

#define PEEKB( s,o ) (*(BYTE far*)MK_FP( s,o ))
#define PEEKW( s,o ) (*(WORD far*)MK_FP( s,o ))
#define PEEKDW( s,o ) (*(DWORD far*)MK_FP( s,o ))

#define POKEB( s,o,v ) (*(BYTE far*)MK_FP( s,o )=v)
#define POKEW( s,o,v ) (*(WORD far*)MK_FP( s,o )=v)
#define POKEDW( s,o,v ) (*(DWORD far*)MK_FP( s,o )=v)

/* define macros for bit isolation */

#define ISO_BIT_0( f ) ((DWORD)(f) & 0x1L)
#define ISO_BIT_1( f ) (((DWORD)(f) & 0x2L       ) >> 1 )
#define ISO_BIT_2( f ) (((DWORD)(f) & 0x4L       ) >> 2 )
#define ISO_BIT_3( f ) (((DWORD)(f) & 0x8L       ) >> 3 )
#define ISO_BIT_4( f ) (((DWORD)(f) & 0x10L      ) >> 4 )
#define ISO_BIT_5( f ) (((DWORD)(f) & 0x20L      ) >> 5 )
#define ISO_BIT_6( f ) (((DWORD)(f) & 0x40L      ) >> 6 )
#define ISO_BIT_7( f ) (((DWORD)(f) & 0x80L      ) >> 7 )
#define ISO_BIT_8( f ) (((DWORD)(f) & 0x100L     ) >> 8 )
#define ISO_BIT_9( f ) (((DWORD)(f) & 0x200L     ) >> 9 )
#define ISO_BIT_10( f ) (((DWORD)(f) & 0x400L     ) >> 10)
#define ISO_BIT_11( f ) (((DWORD)(f) & 0x800L     ) >> 11)
#define ISO_BIT_12( f ) (((DWORD)(f) & 0x1000L    ) >> 12)
#define ISO_BIT_13( f ) (((DWORD)(f) & 0x2000L    ) >> 13)
#define ISO_BIT_14( f ) (((DWORD)(f) & 0x4000L    ) >> 14)
#define ISO_BIT_15( f ) (((DWORD)(f) & 0x8000L    ) >> 15)
#define ISO_BIT_16( f ) (((DWORD)(f) & 0x10000L   ) >> 16)
#define ISO_BIT_17( f ) (((DWORD)(f) & 0x20000L   ) >> 17)
#define ISO_BIT_18( f ) (((DWORD)(f) & 0x40000L   ) >> 18)
#define ISO_BIT_19( f ) (((DWORD)(f) & 0x80000L   ) >> 19)
#define ISO_BIT_20( f ) (((DWORD)(f) & 0x100000L  ) >> 20)
#define ISO_BIT_21( f ) (((DWORD)(f) & 0x200000L  ) >> 21)
#define ISO_BIT_22( f ) (((DWORD)(f) & 0x400000L  ) >> 22)
#define ISO_BIT_23( f ) (((DWORD)(f) & 0x800000L  ) >> 23)
#define ISO_BIT_24( f ) (((DWORD)(f) & 0x1000000L ) >> 24)
#define ISO_BIT_25( f ) (((DWORD)(f) & 0x2000000L ) >> 25)
#define ISO_BIT_26( f ) (((DWORD)(f) & 0x4000000L ) >> 26)
#define ISO_BIT_27( f ) (((DWORD)(f) & 0x8000000L ) >> 27)
#define ISO_BIT_28( f ) (((DWORD)(f) & 0x10000000L) >> 28)
#define ISO_BIT_29( f ) (((DWORD)(f) & 0x20000000L) >> 29)
#define ISO_BIT_30( f ) (((DWORD)(f) & 0x40000000L) >> 30)
#define ISO_BIT_31( f ) (((DWORD)(f) & 0x80000000L) >> 31)

/* define macros for bit setting */

#define SET_BIT_0( f ) (f |= 0x1        )
#define SET_BIT_1( f ) (f |= 0x2L       )
#define SET_BIT_2( f ) (f |= 0x4L       )
#define SET_BIT_3( f ) (f |= 0x8L       )
#define SET_BIT_4( f ) (f |= 0x10L      )
#define SET_BIT_5( f ) (f |= 0x20L      )
#define SET_BIT_6( f ) (f |= 0x40L      )
#define SET_BIT_7( f ) (f |= 0x80L      )
#define SET_BIT_8( f ) (f |= 0x100L     )
#define SET_BIT_9( f ) (f |= 0x200L     )
#define SET_BIT_10( f ) (f |= 0x400L     )
#define SET_BIT_11( f ) (f |= 0x800L     )
#define SET_BIT_12( f ) (f |= 0x1000L    )
#define SET_BIT_13( f ) (f |= 0x2000L    )
#define SET_BIT_14( f ) (f |= 0x4000L    )
#define SET_BIT_15( f ) (f |= 0x8000L    )
#define SET_BIT_16( f ) (f |= 0x10000L   )
#define SET_BIT_17( f ) (f |= 0x20000L   )
#define SET_BIT_18( f ) (f |= 0x40000L   )
#define SET_BIT_19( f ) (f |= 0x80000L   )
#define SET_BIT_20( f ) (f |= 0x100000L  )
#define SET_BIT_21( f ) (f |= 0x200000L  )
#define SET_BIT_22( f ) (f |= 0x400000L  )
#define SET_BIT_23( f ) (f |= 0x800000L  )
#define SET_BIT_24( f ) (f |= 0x1000000L )
#define SET_BIT_25( f ) (f |= 0x2000000L )
#define SET_BIT_26( f ) (f |= 0x4000000L )
#define SET_BIT_27( f ) (f |= 0x8000000L )
#define SET_BIT_28( f ) (f |= 0x10000000L)
#define SET_BIT_29( f ) (f |= 0x20000000L)
#define SET_BIT_30( f ) (f |= 0x40000000L)
#define SET_BIT_31( f ) (f |= 0x80000000L)

/* define macros for bit clearing */

#define CLR_BIT_0( f ) (f &= (0xFFFFFFFF-0x1)        )
#define CLR_BIT_1( f ) (f &= (0xFFFFFFFF-0x2L)       )
#define CLR_BIT_2( f ) (f &= (0xFFFFFFFF-0x4L)       )
#define CLR_BIT_3( f ) (f &= (0xFFFFFFFF-0x8L)       )
#define CLR_BIT_4( f ) (f &= (0xFFFFFFFF-0x10L)      )
#define CLR_BIT_5( f ) (f &= (0xFFFFFFFF-0x20L)      )
#define CLR_BIT_6( f ) (f &= (0xFFFFFFFF-0x40L)      )
#define CLR_BIT_7( f ) (f &= (0xFFFFFFFF-0x80L)      )
#define CLR_BIT_8( f ) (f &= (0xFFFFFFFF-0x100L)     )
#define CLR_BIT_9( f ) (f &= (0xFFFFFFFF-0x200L)     )
#define CLR_BIT_10( f ) (f &= (0xFFFFFFFF-0x400L)     )
#define CLR_BIT_11( f ) (f &= (0xFFFFFFFF-0x800L)     )
#define CLR_BIT_12( f ) (f &= (0xFFFFFFFF-0x1000L)    )
#define CLR_BIT_13( f ) (f &= (0xFFFFFFFF-0x2000L)    )
#define CLR_BIT_14( f ) (f &= (0xFFFFFFFF-0x4000L)    )
#define CLR_BIT_15( f ) (f &= (0xFFFFFFFF-0x8000L)    )
#define CLR_BIT_16( f ) (f &= (0xFFFFFFFF-0x10000L)   )
#define CLR_BIT_17( f ) (f &= (0xFFFFFFFF-0x20000L)   )
#define CLR_BIT_18( f ) (f &= (0xFFFFFFFF-0x40000L)   )
#define CLR_BIT_19( f ) (f &= (0xFFFFFFFF-0x80000L)   )
#define CLR_BIT_20( f ) (f &= (0xFFFFFFFF-0x100000L)  )
#define CLR_BIT_21( f ) (f &= (0xFFFFFFFF-0x200000L)  )
#define CLR_BIT_22( f ) (f &= (0xFFFFFFFF-0x400000L)  )
#define CLR_BIT_23( f ) (f &= (0xFFFFFFFF-0x800000L)  )
#define CLR_BIT_24( f ) (f &= (0xFFFFFFFF-0x1000000L) )
#define CLR_BIT_25( f ) (f &= (0xFFFFFFFF-0x2000000L) )
#define CLR_BIT_26( f ) (f &= (0xFFFFFFFF-0x4000000L) )
#define CLR_BIT_27( f ) (f &= (0xFFFFFFFF-0x8000000L) )
#define CLR_BIT_28( f ) (f &= (0xFFFFFFFF-0x10000000L))
#define CLR_BIT_29( f ) (f &= (0xFFFFFFFF-0x20000000L))
#define CLR_BIT_30( f ) (f &= (0xFFFFFFFF-0x40000000L))
#define CLR_BIT_31( f ) (f &= (0xFFFFFFFF-0x80000000L))

/* define some misc. functions */

#define SIGNAL_ON_INTERVAL( t,i ) ( ( !( (DWORD)t % i ) )?(1):(0))
#define _SET( i ) ( i = 1 )
#define _RESET( i ) ( i = 0 )

#define _INVERT( i ) {                         \
							 if( i )                  \
							  i = 0;                  \
							 else                     \
							  i = 1;                  \
						   }



#endif

/**************************************************************************/

